"""Python SVN Bot

The name is sort of a misnomer because it doesn't really need to be SVN, it in
fact doesn't even have to involve any VCS at all, because what the bot does is
open a UNIX socket which you decide where it should be, and then it just
echoes out whatever is received on that socket.
"""

import threading

def start_thread(f, daemon=False, start=True, *a, **kw):
    t = threading.Thread(target=f)
    t.setDaemon(daemon)
    if start:
        t.start()
    return t



#!/usr/bin/env python

import sys
import subprocess
import socket

# {{{ Utilities
def run(args):
    p = subprocess.Popen(args, stdout=subprocess.PIPE)
    if p.wait() != 0:
        sys.exit(1)
    return p.stdout.read()

class Writer(object):
    def __init__(self, remote_address):
        self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.socket.connect(remote_address)

    def __call__(self, data):
        self.socket.send(data + "\n")
# }}}

# {{{ Get pysvnbot settings.
from pysvnbot import conf
write = Writer(conf.ipc_socket)
# }}}

# {{{ Get SVN info
repos, rev = sys.argv[1:]
author = run(("/usr/bin/svnlook", "author", "--revision", rev, repos)).split("\n", 1)[0]
log = run(("/usr/bin/svnlook", "log", "--revision", rev, repos)).rstrip()
changed = run(("/usr/bin/svnlook", "changed", "--revision", rev, repos)).rstrip().split("\n")
changed_dirs = run(("/usr/bin/svnlook", "dirs-changed", "--revision", rev, repos)).rstrip().split("\n")
# }}}

# {{{ Write out stuff
# Yeah, ugly, so sue me.
write("%s: %s commits r%s. %d file%s in %d dir%s changed." % (repos, author, rev, len(changed), "s"[len(changed) == 1:], len(changed_dirs), "s"[len(changed_dirs) == 1:]))

if len(changed) <= 5:
    for change in changed:
        write("  * %s" % change)

log = log.split("\n")
while log:
    line = log.pop(0)
    while line:
        write(line[:80])
        line = line[80:]
# }}}



#!/usr/bin/env python
"""Main bot module, run it to get the bot to start. Uses extreme amounts
of threads."""

import sys
from pysvnbot import ipc, conf, start_thread

try:
    import pysimpirc
except ImportError, e:
    print >>sys.stderr, ("Failed importing pysimpirc. It is a required "
        "dependency, and can be installed by running `easy_install pysimpirc`"
        "as root or equivalent.")
    print >>sys.stderr, "Error was:", e
    sys.exit(1)

class AutoJoiner(pysimpirc.CommonIRCConnection):
    """Normal IRC bot that tries to autojoin all channels in sequence given
    in __init__.
    """

    def __init__(self, autojoin=(), **kw):
        super(AutoJoiner, self).__init__(**kw)
        self.autojoin_channels = autojoin
        self.autojoined = False

    def autojoin(self):
        if not self.autojoined:
            self.join(self.autojoin_channels)
            self.autojoined = True

    def irc_001(self, prefix, args):
        self.autojoin()

    def irc_005(self, prefix, args):
        self.autojoin()

    def irc_376(self, prefix, args):
        self.autojoin()

    def irc_422(self, prefix, args):
        self.autojoin()

def start_irc_thread(*a, **kw):
    i = AutoJoiner(*a, **kw)
    i.register()  # Register to the IRCd.
    t = start_thread(i.process_forever, start=False)
    t.connection = i  # Hacky? Yeah.
    t.start()
    return t

def main():
    # Start the IPC server first in case the address is taken. (I am aware of
    # the fact that this is a very crappy line, I find it so crappy it's
    # funny.)
    ipc_svr = ipc.IPCServer(lambda m: [[[t.connection.privmsg(c, l) for l in m.replace("\r", "\n").split("\n")] for c in t.connection.autojoin_channels] for t in threads], conf.ipc_socket)
    # Start the IRC bot threads, nothing odd.
    threads = [start_irc_thread(remote_address=remote_address, **conf.ircs[remote_address]) for remote_address in conf.ircs]
    # "Threadize" the main loop of the IPC server as well, then
    # wait for the boats to drift ashore.
    [thread.join() for thread in (threads + [start_thread(ipc_svr.serve_forever, daemon=True)])]

if __name__ == "__main__":
    main()



"""Configuration module for pysvnbot."""

# Where should the IPC socket reside?
ipc_socket = "/tmp/pysvnbot-ipc.sock"

# Which servers and channels should get the messages?
# key = remote address, value = passed to the IRC class' constructor by **d.
ircs = {
    ("irc.lericson.se", 6668): {"nickname": "pysvnbot-test",
        "username": "pysvnbot", "realname": "HAL9000", "autojoin": ("#test",)},
    ("irc.oftc.net", 6667): {"nickname": "pysvnbot-test",
        "username": "pysvnbot", "realname": "HAL9000",
        "autojoin": ("#toxik.fanclub",)}
}



#!/usr/bin/env python
from distutils.core import setup
from sys import version

trove = filter(bool, """
Development Status :: 3 - Alpha
Environment :: Other Environment
Intended Audience :: Developers
Intended Audience :: System Administrators
Operating System :: POSIX
Programming Language :: Python
Topic :: Communications :: Chat :: Internet Relay Chat
Topic :: Software Development :: Version Control
Topic :: System :: Monitoring
Topic :: Software Development :: Build Tools
""".splitlines())

if version < "2.2.3":
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None

setup(name="pysvnbot", version="0.0.3", description="Subversion commit log tracking bot",
    author="Ludvig Ericson", author_email="ludvig.ericson@gmail.com",
    url="http://lericson.se/", packages=["pysvnbot"], classifiers=trove)



"""Inter-process communication interface module."""

import SocketServer

# I think this is obvious enough to not require any documentation, but I might
# be wrong.
class IPCServer(SocketServer.ThreadingUnixStreamServer):
    def __init__(self, callback, server_address):
        self.callback = callback
        SocketServer.ThreadingUnixStreamServer.__init__(self, server_address,
            IPCRequestHandler)

class IPCRequestHandler(SocketServer.StreamRequestHandler):
    """Simple request handler which just reads in rfile and passes the data
    on to the parent servers' callback function.
    """

    def handle(self):
        self.wfile.close()
        self.server.callback(self.rfile.read())
        self.rfile.close()
